package org.glassfish.jta.admin.cli;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.glassfish.api.ActionReport;
import org.glassfish.api.I18n;
import org.glassfish.api.Param;
import org.glassfish.api.admin.AdminCommand;
import org.glassfish.api.admin.AdminCommandContext;
import org.glassfish.api.admin.ExecuteOn;
import org.glassfish.api.admin.RuntimeType;
import org.glassfish.config.support.CommandTarget;
import org.glassfish.config.support.TargetType;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.component.PerLookup;

import com.sun.enterprise.config.serverbeans.Server;
import com.sun.enterprise.config.serverbeans.Servers;
import com.sun.enterprise.transaction.api.JavaEETransactionManager;
import com.sun.enterprise.transaction.api.TransactionAdminBean;
import com.sun.enterprise.util.i18n.StringManager;
import com.sun.logging.LogDomains;

@Service(name = "list-transactions")
@TargetType({ CommandTarget.DAS, CommandTarget.STANDALONE_INSTANCE, CommandTarget.CLUSTERED_INSTANCE })
@ExecuteOn(RuntimeType.DAS)
@Scoped(PerLookup.class)
@I18n("list.transactions")
public class ListTransactions implements AdminCommand {

    static StringManager localStrings =
            StringManager.getManager(ListTransactions.class);

    static Logger _logger = LogDomains.getLogger(ListTransactions.class, LogDomains.JTA_LOGGER);

    @Inject
    JavaEETransactionManager transactionMgr;

    @Inject
    Servers servers;

    @Param(name="detail", optional = true, defaultValue="false")
    Boolean isDetail;
    
    @Param(name="target", alias = "destination", primary = true)
    String destinationServer;

    @Override
    public void execute(AdminCommandContext context) {
        final ActionReport report = context.getActionReport();

        String error = validate(destinationServer);
        if (error != null) {
            _logger.log(Level.WARNING, localStrings.getString("list.transactions.failed"));
            report.setMessage(error);
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        try {
            @SuppressWarnings("unchecked")
            ArrayList<TransactionAdminBean> activeTrans = transactionMgr.getActiveTransactions();

            if (activeTrans == null || activeTrans.size() == 0) {
                if ("OFF".equalsIgnoreCase(getMonitorTransactionLevel(destinationServer))) {
                    report.setMessage(localStrings.getString("list.transactions.logLevelIsOff"));
                    report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                    return;
                }

                report.setMessage(localStrings.getString("list.transactions.noTransactionToList", destinationServer));
                report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
                return;
            }

            report.setMessage(TransactionsInfo.format(activeTrans, isDetail));
            report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        } catch (Exception e) {
            report.setMessage(localStrings.getString("list.transactions.failed"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            report.setFailureCause(e);
        }
    }

    private String validate(String destinationServer) {

        if (servers.getServer(destinationServer) == null) {
            return localStrings.getString("list.transactions.targetServerIsNotKnown", destinationServer);
        }

        if (!isServerRunning(destinationServer)) {
            return localStrings.getString("list.transactions.targetServerIsNotAlive", destinationServer);
        } 

        return null;
    }

    private boolean isServerRunning(String serverName) {
        boolean isRunning = false;
        for (Server server : servers.getServer()) {
            if (serverName.equals(server.getName())) {
                isRunning = server.isRunning();
                break;
            }
        }

        return isRunning;
    }

    private String getMonitorTransactionLevel(String serverName) {
        Server targetServer = null;

        for (Server server : servers.getServer()) {
            if (serverName.equals(server.getName())) {
                targetServer = server;
                break;
            }
        }

        if (targetServer == null) {
            return "OFF";
        }

        return targetServer.getConfig().getMonitoringService().getModuleMonitoringLevels().getTransactionService();
    }

    static class TransactionsInfo {

        private static final String LINE_SEPARATOR       = System.getProperty("line.separator");
        private static final String RESULT_PROMPT        = "List of the transactions:";
        private static final String SUBFIELD             = "----------------------------------------------------------------------------";

        private static final String NO                   = "No.";
        private static final String ID                   = "ID";
        private static final String STATUS               = "Status";

        private static final String ID_TITLE             = "ID             :";
        private static final String STATUS_TILE          = "Status         :";
        private static final String ELAPSED_TIME_TITLE   = "ElapsedTime(ms):";
        private static final String COMPONENT_NAME_TITLE = "ComponentName  :";
        private static final String RESOURCE_NAMES_TITLE = "ResourceNames  :";

        public static String format(ArrayList<TransactionAdminBean> activeTrans, Boolean isDetail) {
            if(isDetail) {
                return formatDetail(activeTrans);
            }else{
                return formatSummary(activeTrans);
            }
        }

        private static String formatSummary(ArrayList<TransactionAdminBean> activeTrans) {
            Collections.sort(activeTrans, new Comparator<TransactionAdminBean>() {

                @Override
                public int compare(TransactionAdminBean tBean1, TransactionAdminBean tBean2) {
                    return tBean1.getId().compareTo(tBean2.getId());
                }
            });

            int digitOfTransSize = getDigitOfNum(activeTrans.size());
            int maxLengthOfNo = (NO.length() < digitOfTransSize ? digitOfTransSize : NO.length()) + 1;
            int maxLengthOfID = getMaxLengthOfID(activeTrans);

            StringBuffer sb = new StringBuffer();
            sb.append(RESULT_PROMPT + LINE_SEPARATOR);
            sb.append(String.format("%-" + maxLengthOfNo + "s", NO));
            sb.append(String.format("%-" + maxLengthOfID + "s", ID));
            sb.append(STATUS);

            for (int i = 0; i < activeTrans.size(); i++) {
                TransactionAdminBean tBean = activeTrans.get(i);

                sb.append(LINE_SEPARATOR);
                sb.append(String.format("%-" + maxLengthOfNo + "s", i));
                sb.append(String.format("%-" + maxLengthOfID + "s", tBean.getId()));
                sb.append(tBean.getStatus());
            }

            return sb.toString();
        }

        private static int getDigitOfNum(int num) {
            return String.valueOf(num).length();
        }

        private static int getMaxLengthOfID(ArrayList<TransactionAdminBean> activeTrans) {
            int maxLength = ID_TITLE.length();
            for (TransactionAdminBean tBean : activeTrans) {
                if (tBean.getId().length() > maxLength) {
                    maxLength = tBean.getId().length();
                }
            }

            return maxLength + 1;
        }

        private static String formatDetail(ArrayList<TransactionAdminBean> activeTrans) {
            Collections.sort(activeTrans, new Comparator<TransactionAdminBean>() {

                @Override
                public int compare(TransactionAdminBean tBean1, TransactionAdminBean tBean2) {
                    return tBean1.getId().compareTo(tBean2.getId());
                }
            });
            
            StringBuffer sb = new StringBuffer();
            sb.append(RESULT_PROMPT + LINE_SEPARATOR);
            sb.append(SUBFIELD);
            for (int i = 0; i < activeTrans.size(); i++) {
                TransactionAdminBean tBean = activeTrans.get(i);

                sb.append(LINE_SEPARATOR + "[No " + (i + 1) + "]");
                sb.append(LINE_SEPARATOR + ID_TITLE + tBean.getId());
                sb.append(LINE_SEPARATOR + STATUS_TILE + tBean.getStatus());
                sb.append(LINE_SEPARATOR + ELAPSED_TIME_TITLE + tBean.getElapsedTime());
                sb.append(LINE_SEPARATOR + COMPONENT_NAME_TITLE + tBean.getComponentName());
                sb.append(LINE_SEPARATOR + RESOURCE_NAMES_TITLE + parseResourceNames(tBean.getResourceNames()));
            }
            sb.append(LINE_SEPARATOR + SUBFIELD);

            return sb.toString();
        }

        private static String parseResourceNames(ArrayList<String> resourceNames) {
            StringBuffer sb = new StringBuffer();
            if (resourceNames != null && resourceNames.size() != 0) {
                for (int i = 0; i < resourceNames.size(); i++) {
                    sb.append(resourceNames.get(i));
                    if (i != resourceNames.size() - 1) {
                        sb.append(", ");
                    }
                }
            }
            
            return sb.toString();
        }

    }

}
